%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% nf-graph.pl 	
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% comprova si un terme o llista de termes est en forma normal 
% (el terme Term est renombrat --> ground).
% Per tant considerem FN si ho est per alguna substituci

normal_form(_,Term):- is_variable(Term),!.
normal_form([],_):-!.
normal_form([[L,_]|Rest],Term):-
	normal_form1(L,Term),!, 
	normal_form(Rest,Term),!.

normal_form1(_,Term):- is_variable(Term),!.
normal_form1(_,[]):-!.
normal_form1(L,[X|R]):-!,normal_form1(L,X),normal_form1(L,R),!.
normal_form1(L,Term):- match0(Term,L),!,fail.
normal_form1(L,Term):- Term=..[_|Args],
		       normal_form1(L,Args),!.

match0(Term,L):- refreshterm(L,L1),!, unifica(Term,L1),!.


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Elimina els cicles insatisfactibles per innermost (nf) (rewrite-step).
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

sel_and_sat_nf(Rules,Lc,Lcrest):-
	sele(nf,Lc,Lcnf,Lcrest),
	del_repes(Lcnf,Lcnf1),
	sat_nf(Rules,Lcnf1),!.

sele(_,[],[],[]):-!.
sele(Q,[T|R],[T|Rnf],Rest):-T=..[Q|_],!,
	sele(Q,R,Rnf,Rest),!.
sele(Q,[X|R],Rnf,[X|Rest]):-
	sele(Q,R,Rnf,Rest),!.

sat_nf(_,[]):-!.
sat_nf(Rules,[nf(X)|Rnf]):-
	normal_form(Rules,X),!,
	sat_nf(Rules,Rnf),!.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Per tots els S>=T que hi ha a Lc, si hi ha un nic redex amb una
% nica regla aplicable--> es fa el pas de reescriptura.
%   (no considero cap estratgia left-most,.... )
% Si no es pot aplicar cap regla(considerant unificaci, no matching)
% llavors fracassa--> es llenar el cicle.

try_rewrite_step(Rules,Lc,LcOut):-
	sele_ge_left_novar(Lc,Lc1,Lcrest),
	del_repes(Lc1,Lc2),!,
	try_rewrite_step1(Rules,Lc2,Lc3),
	concat312(Lc3,Lcrest,LcOut),!.

sele_ge_left_novar([],[],[]):-!.
sele_ge_left_novar([S>=T|R],[S>=T|R1],Rest):-
	\+is_variable(S),!,
	sele_ge_left_novar(R,R1,Rest),!.
sele_ge_left_novar([X|R],R1,[X|Rest]):-
	sele_ge_left_novar(R,R1,Rest),!.


try_rewrite_step1(_,[],[]):-!.
try_rewrite_step1(Rules,[L>=_|_],_):-
	conv_cts_vars(L,L1),
	no_rule_aplicable(Rules,L1),!,fail.
try_rewrite_step1(Rules,[L>=R|Lc],[L1>=R|LcOut]):-
	rewrite_step(Rules,L,L1),!,
	try_rewrite_step1(Rules,Lc,LcOut),!.
try_rewrite_step1(Rules,[Q|Lc],[Q|LcOut]):-
	try_rewrite_step1(Rules,Lc,LcOut),!. %ms d'un redex o ms d'una regla aplic.
					   % per tant, no fem pas reescriptura.

% comprova si un terme o llista de termes est en forma normal per quals. substituci
% (el terme Term cont variables).
no_rule_aplicable(_,Term):- var(Term),!.
no_rule_aplicable([],_):-!.
no_rule_aplicable([[L,_]|Rest],Term):-
		no_rule_aplicable1(L,Term),!, 
		no_rule_aplicable(Rest,Term),!.

no_rule_aplicable1(_,Term):- var(Term),!.
no_rule_aplicable1(_,[]):-!.
no_rule_aplicable1(L,[X|R]):-!,no_rule_aplicable1(L,X),no_rule_aplicable1(L,R),!.
no_rule_aplicable1(L,Term):- match2(Term,L),!,fail.
no_rule_aplicable1(L,Term):- Term=..[_|Args],no_rule_aplicable1(L,Args),!.

match2(Term,L):-refreshterm(Term,Term1), refreshterm(L,L1),!, 
		unifica(Term1,L1),!.

%%%%%
 
rewrite_step(Rules,Term,TermOut):-
	obtain_redex(Rules,Term,Term1),  % si redex nic s Term1.
	Term1=..[F|_],
	obtain_rules_defF(Rules,F,RulesF),
	try_only_one(Term1,RulesF,Term2),!,
	replace_all(Term1,Term2,Term,TermOut),!.
rewrite_step(_,Term,Term). %  si fracassa obtain_redex (no nic)
			   %  o b try_only_one(cap o ms d'una regla aplic)
			   %  retornem el mateix terme

%comprova nic redex i el retorna a TermOut
obtain_redex(Rules,Term,Term):-  %  redex al top. Arguments en forma-normal per quals. substituci
 	Term=..[_|Args],	 %  Si no pas aplicable ja fallar try_only_one
	conv_cts_vars(Args,Args1),  % a sota no hi ha pas possible per quals. substitucio
	no_rule_aplicable(Rules,Args1),!.
obtain_redex(Rules,Term,TermOut):-    % per alguna subst. hi ha possible redex en algun argument
	Term=..[_|Args],
	obtain_unique_redex(Rules,Args,TermOut),!.

obtain_unique_redex(Rules,[Arg|Rest],TermOut):- 
	conv_cts_vars(Arg,Arg1),   % el primer argument esta en FN per quals. substituci
	no_rule_aplicable(Rules,Arg1),!,
	obtain_unique_redex(Rules,Rest,TermOut),!.
obtain_unique_redex(Rules,[Arg|Rest],TermOut):- % hi ha rule aplicable a Arg
	conv_cts_vars(Rest,Rest1), % la resta d'arguments estan en FN per quals. substituci
	no_rule_aplicable(Rules,Rest1),		
	obtain_redex(Rules,Arg,TermOut),!. % obtenim redex nic a Arg.


%Si cap o ms d'una regla aplicable fracassa (unificaci)
%Si una regla aplicable per pas de reescriptura noms per determinades substituci fracassa
%si noms una regla aplicable i pas reescriptura nic (per quals. subst)
%   retorna el terme resultant de fer el pas de reescriptura (matching)
%(Term i R sn ground)
try_only_one(Term,Rules,R):-
	conv_cts_vars(Term,Term1),
	obtain_rules_aplic_top(Term1,Rules,[Rule]), % una nica regla aplicable a top (per quals. subst)
	refreshterm(Rule,[L,R]),
	unifica(Term,L),!.   % fa matching ja que Term s ground

% obt totes les regles aplicables (per alguna substituci) a posici top de Term. (unifica)
obtain_rules_aplic_top(_,[],[]).
obtain_rules_aplic_top(Term,[[L,R]|Rules],[[L,R]|RulesAplic]):-
	match2(Term,L),!,
	obtain_rules_aplic_top(Term,Rules,RulesAplic),!.
obtain_rules_aplic_top(Term,[_|Rules],RulesAplic):-
	obtain_rules_aplic_top(Term,Rules,RulesAplic),!.



obtain_rules_defF([],_,[]):-!.
obtain_rules_defF([[L,R]|Rules],F,[[L,R]|RulesF]):-
	functor(L,F,_),!,
	obtain_rules_defF(Rules,F,RulesF),!.
obtain_rules_defF([_|Rules],F,RulesF):-
	obtain_rules_defF(Rules,F,RulesF),!.


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Elimina les regles inaplicables segons estratgies inner
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

simplif_rules_strat(Rules,RulesOut):-
	simplif_rules_inner(Rules,RulesOut),!,
	(
	 Rules==RulesOut
	 ;
	 inici(RulesOut)
	),!.


% comprova si un terme o llista de termes est en forma normal 
% (el terme Term cont variables).
r_normal_form(_,Term):- var(Term),!.
r_normal_form([],_):-!.
r_normal_form([[L,_]|Rest],Term):-
		r_normal_form1(L,Term),!, 
		r_normal_form(Rest,Term),!.

r_normal_form1(_,Term):- var(Term),!.
r_normal_form1(_,[]):-!.
r_normal_form1(L,[X|R]):-!,r_normal_form1(L,X),r_normal_form1(L,R),!.
r_normal_form1(L,Term):- match1(Term,L),!,fail.
r_normal_form1(L,Term):- Term=..[_|Args],r_normal_form1(L,Args),!.

match1(Term,L):-renvars(Term,Term1), refreshterm(L,L1),!, unifica(Term1,L1),!.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Elimina les regles inaplicables segons inner

simplif_rules_inner(Rules,Rules):- \+(innermost),!.
simplif_rules_inner(Rules,RulesOut):-
	simplif_rules_inner(Rules,Rules,RulesOut),!.

simplif_rules_inner([],_,[]):-!.
simplif_rules_inner([[L,R]|Rules],Rules0,[[L,R]|RulesOut]):-
	add_inn(L,[],Lnf),
	sat_rnf(Rules0,Lnf),!,
	simplif_rules_inner(Rules,Rules0,RulesOut),!.
simplif_rules_inner([_|Rules],Rules0,RulesOut):-
	simplif_rules_inner(Rules,Rules0,RulesOut),!.


sat_rnf(_,[]):-!.
sat_rnf(Rules,[nf(X)|Rnf]):-
	r_normal_form(Rules,X),!,
	sat_rnf(Rules,Rnf),!.




%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% S'executa quan es mostra la demostraci (CAS INNERMOST)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%LcancelNF=[[N1,N2,nf(X)],...]
obtain_graph_and_cancelled_connexions(Rules,Qabs,SubGraphs,LcancelNF):-
	build_graph_cancelled_conn0(Rules,Qabs,Qabs,Graph,LcancelNF),
	simplify_graph(Graph,SubGraphs),!.

build_graph_cancelled_conn0(Rules,Qabs,AllQ,Graph,LcancelNF):-
	innermost,!,
	build_graph_cancelled_conn(Rules,Qabs,AllQ,Graph,[],LcancelNF),!.
build_graph_cancelled_conn0(Rules,Qabs,AllQ,Graph,[]):-
	build_graph_conn(Rules,Qabs,AllQ,Graph),!.

build_graph_cancelled_conn(_,[],_,[],L,L).
build_graph_cancelled_conn(Rules,[q(N,Q,Lc)|Qabs],AllQ,[[q(N,Q,Lc),LM]|Graph],Lcan1,LcanOut):-
	findall(Noconn,nosatNF_connect_graph(Rules,N,Q,AllQ,Noconn),Lcan2),
	findall(M,connect_graph(Rules,Q,AllQ,M),LM),
	concat312(Lcan1,Lcan2,Lcan3),
	build_graph_cancelled_conn(Rules,Qabs,AllQ,Graph,Lcan3,LcanOut),!.

nosatNF_connect_graph(Rules,N,Q1,[q(M,Q2,_)|_],[N,M,X]):-
	Q1=..[_|[_,R]],
	Q2=..[_|[L,_]],
	refreshterm(R,R1),
	refreshterm(L,L1),
	unify0(R1,L1), 
	check_some_arg_no_nf(Rules, L1,X).	
nosatNF_connect_graph(Rules,N,Q1,[_|Rest],NoConn):-
	nosatNF_connect_graph(Rules,N,Q1,Rest,NoConn).

check_some_arg_no_nf(Rules,T,X):-
	T=..[_|Args],
	remove_variables_and_renvars(Args,[],ArgsNF),
	some_nf_nosat(Rules,ArgsNF,X),!.		

some_nf_nosat(Rules,Lc,X):-
	member22(nf(X),Lc),
	\+(normal_form(Rules,X)),!.


build_graph_conn(_,[],_,[]).
build_graph_conn(Rules,[q(N,Q,Lc)|Qabs],AllQ,[[q(N,Q,Lc),LM]|Graph]):-
	findall(M,connect_graph(Rules,Q,AllQ,M),LM),
	build_graph_conn(Rules,Qabs,AllQ,Graph),!.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

obtain_cancel_and_step_each_subgraph(_,_,[],[],[],[],_):-!.
obtain_cancel_and_step_each_subgraph(Rules,Def,[Sub|Subgraphs],LcancelNF,LcancelNoStep,LcycStep,Lnums):-
	obtain_cancel_and_step_1_subgraph(Rules,Def,Sub,LcancelNF1,LcancelNoStep1,LcycStep1,Lnums),
	obtain_cancel_and_step_each_subgraph(Rules,Def,Subgraphs,LcancelNF2,LcancelNoStep2,LcycStep2,Lnums),
	concat312(LcancelNF1,LcancelNF2,LcancelNF),
	concat312(LcancelNoStep1,LcancelNoStep2,LcancelNoStep),
	concat312(LcycStep1,LcycStep2,LcycStep),!.

obtain_cancel_and_step_1_subgraph(Rules,Def,Graph,LcancelNF,LcancelNoStep,LcycStep,Lnums):-
	obtain_simple_cycles_from_graph(Graph,NumsCycles),
	obtain_no_real_cycles(Rules,Def,Graph,NumsCycles,NumsCycles1,LcancelNF,LcancelNoStep),
	( (innermost,!,
	   findall(N,member22([q(N,_>_,_),_],Graph),LStricts),
	   obtain_new_subgraphs(LStricts,NumsCycles1,LSUBCycles),
	   obtain_step_modified_cycles(Rules,Graph,LSUBCycles,LcycStep,Lnums)
	  )
	  ;
	  (LcycStep=[])
	),!.	
	

%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%

%LcancelNF=[[Nums1,Term1],[Nums2,Term2],...] ground
%LcancelNoStep=[[K,Nums1,L1>=R1],[K,Nums2,L2>=R2],...] ground
% K=s1 no cicle per constructors
% K=s2 no cicle per no possible rewrite step
% K=s3 no cicle per pas unic i despres no per constructor :[K,Nums1,[L,L1>=R1]]
obtain_no_real_cycles(Rules,Def,G,NumsCycles,NumsCyclesOut,LcancelNF,LcancelNoStep):-
	selec_nodes_several_cycles(NumsCycles,Nodes),
	delete33(NumsCyc1,NumsCycles,NumsCycles1),
	check_WHY_no_real_cycle(Rules,Def,G,Nodes,NumsCyc1,[K,X]),!,
	obtain_no_real_cycles(Rules,Def,G,NumsCycles1,NumsCyclesOut,LcancelNF1,LcancelNoStep1),
	( ( K=nf,!, LcancelNF=[[NumsCyc1,X]|LcancelNF1], LcancelNoStep=LcancelNoStep1) 
	  ;
	  (LcancelNF=LcancelNF1,LcancelNoStep=[[K,NumsCyc1,X]|LcancelNoStep1])
	),!.
obtain_no_real_cycles(_,_,_,NumsCycles,NumsCycles,[],[]).


check_WHY_no_real_cycle(Rules,Def,G,Nodes,NumsCycle,[K,X]):-
	perform_cycle(G,Nodes,NumsCycle,Conditions),  %Conditions es ground
	(
	 (some_nf_nosat(Rules,Conditions,X),!,K=nf)
	 ;
	 (sele_ge_left_novar(Conditions,Cond,_),
	  del_repes(Cond,Cond1),
	  try_no_possible_step0(Rules,Def,Cond1,X,K))
	),!.	
		

try_no_possible_step0(_,Def,Lc,X,'s1'):-
	try_reach_unsat(Lc,Def,X),!.
try_no_possible_step0(Rules,_,Lc,X,'s2'):-  % no hi ha cap regla aplicable 
	innermost,			    % per quals. substituci
	try_no_possible_step(Rules,Lc,X),!.
try_no_possible_step0(Rules,Def,Lc,[L,X],'s3'):-
	innermost,
	member22(L>=R,Lc),
	try_rewrite_step1(Rules,[L>=R],[L1>=R]),
	\+(L==L1),
	try_reach_unsat([L1>=R],Def,X),!.


try_reach_unsat(Lc,Def,L>=R):-
	member22(L>=R,Lc),
	check_no_reachability(L,R,Def),!.
 
check_no_reachability(L,R,Def):-
	check_reachability1(L,R,Def),!,fail.
check_no_reachability(_,_,_).

try_no_possible_step(Rules,Lc,L>=R):-
	member22(L>=R,Lc),
	conv_cts_vars(L,L1),
	no_rule_aplicable(Rules,L1),!.	
		

%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%

obtain_step_modified_cycles(_,_,[],[],_):-!.
obtain_step_modified_cycles(Rules,Graph,[SUBCycles|LSUBCycles],LcycStep,Lnums):-
	l_union(SUBCycles,Nums),
	inter(Nums,Lnums,[_|_]),!,
	obtain_step_modified_cycle1(Rules,Graph,SUBCycles,LcycStep1),
	obtain_step_modified_cycles(Rules,Graph,LSUBCycles,LcycStep2,Lnums),
	concat312(LcycStep1,LcycStep2,LcycStep),!.
obtain_step_modified_cycles(Rules,Graph,[_|LSUBCycles],LcycStep,Lnums):-
	obtain_step_modified_cycles(Rules,Graph,LSUBCycles,LcycStep,Lnums),!.

 % un sol cicle
obtain_step_modified_cycle1(Rules,Graph,[Nums],[[Nums,LCond,LCondNew]]):-
	perform_cycle(Graph,[],Nums,Conditions),
	sele_ge_left_novar(Conditions,Cond,_),
	del_repes(Cond,LCond),
	proof_try_rewrite_step(Rules,LCond,LCondNew),
	LCondNew=[_|_],!.
obtain_step_modified_cycle1(_,_,[_],[]):-!.
% varis cicles
obtain_step_modified_cycle1(Rules,Graph,NumCycles,LcycStep):-
	selec_nodes_several_cycles(NumCycles,Nodes),!,
	obtain_step_modified_cycle2(Rules,Graph,Nodes,NumCycles,LcycStep),!.
	

% segur que almenys una regla aplicable
proof_try_rewrite_step(_,[],[]):-!.
proof_try_rewrite_step(Rules,[L>=R|Lc],[L1>=R|LcOut]):-
	rewrite_step(Rules,L,L1),\+(L=L1),!,
	proof_try_rewrite_step(Rules,Lc,LcOut),!.
proof_try_rewrite_step(Rules,[_|Lc],LcOut):-
	proof_try_rewrite_step(Rules,Lc,LcOut),!. % ms d'un redex o ms d'una regla aplic. o depn subst.
					   	  % per tant, no fem pas reescriptura.

obtain_step_modified_cycle2(_,_,_,[],[]).
obtain_step_modified_cycle2(Rules,Graph,Nodes,[Nums|NumCycles],[[Nums,LCond,LCondNew]|LcycStep]):-
	perform_cycle(Graph,Nodes,Nums,Conditions),
	sele_ge_left_novar(Conditions,Cond,_),
	del_repes(Cond,LCond),
	LCond=[_|_],
	proof_try_rewrite_step(Rules,LCond,LCondNew),
	LCondNew=[_|_],
	obtain_step_modified_cycle2(Rules,Graph,Nodes,NumCycles,LcycStep),!.
obtain_step_modified_cycle2(Rules,Graph,Nodes,[_|NumCycles],LcycStep):-
	obtain_step_modified_cycle2(Rules,Graph,Nodes,NumCycles,LcycStep),!.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

mostrar_noconnexions_nf([]):-!.
mostrar_noconnexions_nf([[N,M,X]|LnoConnNF]):-
	write('- There is no connexion between '), 
	write(N), write(' and '), write(M),write(' because'),nl,
	write('  normal_form('),write(X),write(')  is not true.'),nl,nl,
	mostrar_noconnexions_nf(LnoConnNF),!.

mostrar_cancelats_nf([]):-!.
mostrar_cancelats_nf([[Nums,T]|LcancelNF]):-
	write('- The cycle build with the constraints: { '),mostra_nums_cicles(Nums),write(' }'),nl,
	write('  is cancelled since  normal-form('),write(T),write(')  is not true.'),nl,nl,!,
	mostrar_cancelats_nf(LcancelNF),!.

mostrar_cancelats_noStep([]):-!.
mostrar_cancelats_noStep([['s1',Nums,S>=T]|Lcancel]):-
	write('- The cycle build with the constraints: { '),mostra_nums_cicles(Nums),write(' }'),nl,
	write('  is cancelled since there is no possible rewrite sequente to go '),nl,
	write('          from  '),write(S),write('  to  '),write(T),nl,
	write('  which is required in the condition: '), write(S),write(' ->* '),write(T),nl,nl,!,
	mostrar_cancelats_noStep(Lcancel),!.
mostrar_cancelats_noStep([['s2',Nums,S>=T]|Lcancel]):-
	write('- The cycle build with the constraints: { '),mostra_nums_cicles(Nums),write(' }'),nl,
	write('  is cancelled since it is not possible to produce any rewrite step to the term '),nl,
	tab(10),write(S),nl,
	write('  which is required in the condition: '), write(S),write(' ->* '),write(T),nl,nl,!,
	mostrar_cancelats_noStep(Lcancel),!.
mostrar_cancelats_noStep([['s3',Nums,[S0,S>=T]]|Lcancel]):-
	write('- The cycle build with the constraints: { '),mostra_nums_cicles(Nums),write(' }'),nl,
	write('  is cancelled since the condition '),nl,
	tab(10), write(S0),write(' ->* '),write(T),nl,
	write('  required to build the cycle can be modified by applying a unique rewriting step obtaining'),nl,
	tab(10), write(S),write(' ->* '),write(T),nl,
	write('  and there is no possible rewrite sequente to go from  '),write(S),write('  to  '),write(T),nl,nl,
	mostrar_cancelats_noStep(Lcancel),!.


mostrar_cicles_Step([]):-!.
mostrar_cicles_Step([[Nums,LCond,LCondNew]|LcycStep]):-
	write('- The following condition:'),nl,
	mostra_condicions_cycle(LCond),
	write('  required to build the cycle with the constraints: { '),mostra_nums_cicles(Nums),write(' }'),nl,
	write('  can be modified by applying a unique rewriting step:'),nl,
	show_each_rewrite_step(LCond,LCondNew),nl,nl,!,
	mostrar_cicles_Step(LcycStep),!.

show_each_rewrite_step([],[]):-!.
show_each_rewrite_step([S>=T|Lc],[S1>=T|LcOut]):-!,
	tab(5),write(S),write(' ->* '),write(T),tab(5),
	write('obtaining'),tab(5), write(S1),write(' ->* '),write(T),nl,
	show_each_rewrite_step(Lc,LcOut),!.
		

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%mostra_constraints_cycle(_,[]):-!.
%mostra_constraints_cycle(Q,[N|R]):-
%	member21(q(N,C),Q),
%	tab(5), write(C),nl,
%	mostra_constraints_cycle(Q,R),!.

mostra_condicions_cycle([]):-!.
mostra_condicions_cycle(R):-
	mostra_nf_cycle(R,R1),
	mostra_condicions_ge(R1),!.

mostra_nf_cycle(R,ROut):-
	delete32(nf(T),R,R1),!,
	(
	 ( is_variable(T),! )
	 ;
	 ( tab(5),write('normal-form('),write(T),write(')'),nl)
	),!,
	mostra_nf_cycle(R1,ROut),!.
mostra_nf_cycle(R,R).

mostra_condicions_ge([]):-!.
mostra_condicions_ge([L>=R|Lc]):-
	tab(5),write(L),write(' ->* '),write(R),nl,
	mostra_condicions_ge(Lc),!.
mostra_condicions_ge([_|Lc]):-
	mostra_condicions_ge(Lc),!.
	
