
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% a_intrpo.pl : Funcions auxiliars per int_rpo.pl (03-09-02)
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 
% USES : list, (ordre:is_variable)
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% subterm_possible_segures(L,R,LInts).
% all_variables(L)
% int_arity(N,L)
% list_symbols(Term,L,LOut)
% selecc_elems(Lpos,Args,ArgsOut)
% obtain_vars(Term,LvarsT)
% member_term_igInt(S,T,LInts)
% en_pos_segura_Int(T,S,LInts)
% strict_subt_segura(T,S,LInts)
% top_segur(F,LInts)
% difer_args_Int(StatF,ArgsL,ArgsR,DifLR,DifRL,LInts)
% try_subterm_or_prec_segures(Rules,Prec,PrecOut,Stat,StatOut,LInts)
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%1

% Ja est definit a ordre.pl
%is_variable(V):-variables(Vars), member212(V,Vars).


all_variables([]):-!.
all_variables([X|R]):-is_variable(X), all_variables(R),!.
	


int_arity(0,[[1]]):-!.
int_arity(N,L):-gen_int_subterm(N,L1),
		gen_int_sel_sub(N,L2),
		concat312(L1,L2,L3),
		reverse1(L3,L),!.

% genero totes les possibilitats al revs per evitar concats
% al final faig reverse de tots.
gen_int_subterm(N,L):-gen_int_subterm1(N,L1), map(reverse1,L1,L),!.

gen_int_subterm1(1,[[1,0]]):-!.
gen_int_subterm1(N,[[1|L12]|L11]):-
		N>1, M is N-1,
		gen_int_subterm1(M,L1),
		afegir(L1,0,L11), 
		crearlistNK(N,0,L12),!.

gen_int_sel_sub(N,L):-gen_int_sel_sub1(N,L1), map(reverse1,L1,L),!.

gen_int_sel_sub1(1,[[0,1],[1,1]]):-!.
gen_int_sel_sub1(N,L):-
		N>1, M is N-1,
		gen_int_sel_sub1(M,L1),
		afegir(L1,0,L11), 
		afegir(L1,1,L12),		
		concat312(L11,L12,L),!.


list_symbols([],L,L1):-!,L=L1.
list_symbols([Term|Rest],L,L2):-!,
	list_symbols(Term,L,L1),
	list_symbols(Rest,L1,L2).
list_symbols(Term,L,L):-
	is_variable(Term),!.
list_symbols(Term,L,L1):-
	Term=..[F|Args],
	\+(member212(F,L)),!,
	list_symbols(Args,[F|L],L1).
list_symbols(Term,L,L1):-
	Term=..[_|Args],list_symbols(Args,L,L1).


list_symbolsAr([],L,L1):-!,L=L1.
list_symbolsAr([Term|Rest],L,L2):-!,
	list_symbolsAr(Term,L,L1),
	list_symbolsAr(Rest,L1,L2).
list_symbolsAr(Term,L,L):-
	is_variable(Term),!.
list_symbolsAr(Term,L,L1):-
	Term=..[F|Args],length21(Args,Ar),
	\+(member212([F,Ar],L)),!,
	list_symbolsAr(Args,[[F,Ar]|L],L1).
list_symbolsAr(Term,L,L1):-
	Term=..[_|Args],list_symbolsAr(Args,L,L1).


%selecciona els subtermes de L que es troben en alguna posici de Lpos
selecc_elems(Lpos,L,L1):-
	enumerate(L,L_en),
	findall(S,elemsel(Lpos,L_en,S),L1).

elemsel(Lpos,L,S):- 
	member22([X,S],L), member22(X,Lpos).


%obtain_vars: obt totes les variables d'un terme o llista de termes

obtain_vars(Term,L):-obtain_vars(Term,[],L),!.

obtain_vars([],L,L1):-!,L=L1.
obtain_vars([X|R],L,Lvars):-!,
	obtain_vars(X,L,L1),
	obtain_vars(R,L1,Lvars),!.
obtain_vars(T,L,Lvars):-
	is_variable(T),!,
	addifno(T,L,Lvars),!.
obtain_vars(T,L,Lvars):-
	T=..[_|Args],
	obtain_vars(Args,L,Lvars),!.


	
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% comprova si L i R sn iguals usant Ints niques i treien 
% recursivament subtermes impossibles.
iguals_sota_Int(L,R,LInts):-
	int_terms_under_fixed_ints([L,R],LInts,[L1,R1],LInts1),
	iguals_sota_Int1(L1,R1,LInts1),!.

iguals_sota_Int1(L,R,_):-
	(is_variable(L) ; is_variable(R)),!,L=R.
iguals_sota_Int1(L,L,_):-!.
iguals_sota_Int1(L,R,LInts):-
	L=..[F|ArgsL],
	R=..[F|ArgsR],
	pos_impos(F,LInts,Limpos),
	args_pos_poss(ArgsL,Limpos,A1),
	args_pos_poss(ArgsR,Limpos,A2),
	zip_iguals_sota_Int1(A1,A2,LInts),!.

int_terms_under_fixed_ints([],LInts,[],LInts):-!.
int_terms_under_fixed_ints(L,LInts,LOut,LIntsOut):-
	selecc_fixed_ints(LInts,LIntsOut,Lfixed),
	Lfixed=[_|_],!,
	interp_terms(L,Lfixed,LOut),!.
int_terms_under_fixed_ints(L,LInts,L,LInts).
	

zip_iguals_sota_Int1([],[],_):-!.
zip_iguals_sota_Int1([X|R1],[Y|R2],LInts):-
	iguals_sota_Int1(X,Y,LInts),
	zip_iguals_sota_Int1(R1,R2,LInts).

args_pos_poss(ArgsL,Limpos,A):-
	enumerate(ArgsL,ArgsEnum),
	elim_impos(ArgsEnum,Limpos,A),!.

elim_impos([],_,[]):-!.
elim_impos([[P,_]|Lpossub],Limpos,LpossubOut):-
	member22(P,Limpos),!,
	elim_impos(Lpossub,Limpos,LpossubOut).
elim_impos([[_,S]|Lpossub],Limpos,[S|LpossubOut]):-
	elim_impos(Lpossub,Limpos,LpossubOut).


%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% com member_term1 per amb iguals_sota_Int enlloc de =.


member_term_igInt(S,T,LInts):-
	int_terms_under_fixed_ints([S,T],LInts,[S1,T1],LInts1),
	member_term_igInt1(S1,T1,LInts1),!.
	
member_term_igInt1(X,Y,LInts):-iguals_sota_Int1(X,Y,LInts),!.
member_term_igInt1(X,T,LInts):-T=..[_|Args],member_term_igInt1list(X,Args,LInts).

member_term_igInt1list(X,[T|_],LInts):-member_term_igInt1(X,T,LInts),!.
member_term_igInt1list(X,[_|R],LInts):-member_term_igInt1list(X,R,LInts),!.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% 2n es troba en pos. segura de 1r (desprs d'elim. subt impos)

en_pos_segura_Int(T,S,LInts):-
	int_terms_under_fixed_ints([T,S],LInts,[T1,S1],LInts1),
	en_pos_segura_Int1(T1,S1,LInts1),!.

en_pos_segura_Int1(L,L,_):-!.
en_pos_segura_Int1(L,L1,LInts):-
	iguals_sota_Int1(L,L1,LInts),!.
en_pos_segura_Int1(L,_,_):-
	is_variable(L),!,fail.
en_pos_segura_Int1(L,R,LInts):-
	L=..[F|Args],
	length21(Args,Ar),
	obtain_pos_segures(F,Ar,Lpos,LInts),
	selecc_elems(Lpos,Args,Args1),
	en_pos_segura_Int1_list(Args1,R,LInts).

en_pos_segura_Int1_list([],_,_):-!,fail.
en_pos_segura_Int1_list([X|_],R,LInts):-
	en_pos_segura_Int1(X,R,LInts),!.
en_pos_segura_Int1_list([_|Rest],R,LInts):-
	en_pos_segura_Int1_list(Rest,R,LInts),!.

	
%%%%%%%%%%%%%%

%% comprova 2n arg s subterme estricte de 1r en posici segura 	

strict_subt_segura(T,S,LInts):-
	int_terms_under_fixed_ints([T,S],LInts,[T1,S1],LInts1),
	strict_subt_segura1(T1,S1,LInts1),!.

strict_subt_segura1(Var,_,_):- is_variable(Var),!,fail.
strict_subt_segura1(Term,Term,_):- !,fail.
strict_subt_segura1(Term,Subt,LInts):-
	Term=..[F|Args], top_segur(F,LInts),!,
	length(Args,Ar),
	obtain_pos_segures(F,Ar,Lpos,LInts),
	selecc_elems(Lpos,Args,Args1),!,
	en_pos_segura_Int1_list(Args1,Subt,LInts),!.
strict_subt_segura1(Term,Subt,LInts):-
	Term=..[F|Args], length21(Args,Ar),
	obtain_pos_segures(F,Ar,Lpos,LInts),
	selecc_elems(Lpos,Args,Args1),
	strict_subt_segura1_list(Args1,Subt,LInts).

strict_subt_segura1_list([],_,_):-!,fail.
strict_subt_segura1_list([T|_],Subt,LInts):-
	strict_subt_segura1(T,Subt,LInts),!.
strict_subt_segura1_list([_|Rest],Subt,LInts):-
	strict_subt_segura1_list(Rest,Subt,LInts),!.

%%%%%%%
	
top_segur(F,LInts):-
 	member22([F,_,IntsF],LInts),!,
	segurtop(IntsF),!.
top_segur(_,_):-!.
		
segurtop(IntsF):-
	member22([0|_],IntsF),!,fail.
segurtop(_):-!.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


difer_args_Int(mul,ArgsL,ArgsR,DifLR,DifRL,LInts):-
	difer_Int(ArgsL,ArgsR,DifLR,LInts),
	difer_Int(ArgsR,ArgsL,DifRL,LInts),!.
difer_args_Int(lex,ArgsL,ArgsR,DifLR,DifRL,LInts):-
	difer_lex0(ArgsL,ArgsR,DifLR,DifRL,LInts),!.
difer_args_Int(lexr,ArgsL,ArgsR,DifLR,DifRL,LInts):-
	reverse1(ArgsL,ArgsL1),
	reverse1(ArgsR,ArgsR1),
	difer_lex0(ArgsL1,ArgsR1,DifLR1,DifRL1,LInts),
	reverse1(DifLR1,DifLR),
	reverse1(DifRL1,DifRL),!.
difer_args_Int(cap,ArgsL,ArgsR,DifLR,DifRL,LInts):-
	length21(ArgsL,N), length21(ArgsR,N),
	difer_lex0(ArgsL,ArgsR,DifLR,DifRL,LInts),!.
difer_args_Int(cap,ArgsL,ArgsR,ArgsL,ArgsR,_):-!.

	
difer_Int(L,[],L1,_):-!,L1=L.
difer_Int(L,[X|R],L1,LInts):-
	delete32(Y,L,L11),
	iguals_sota_Int(X,Y,LInts),!, 
	difer_Int(L11,R,L1,LInts),!.
difer_Int(L,[_|R],L1,LInts):-difer_Int(L,R,L1,LInts),!.

difer_lex0([],L,[],L,_):-!.
difer_lex0(L,[],L,[],_):-!.
difer_lex0([X|ArgsL],[Y|ArgsR],DifLR,DifRL,LInts):-
	iguals_sota_Int(X,Y,LInts),!,
	difer_lex0(ArgsL,ArgsR,DifLR,DifRL,LInts),!.
difer_lex0([X|ArgsL],[Y|ArgsR],[X|DifLR],[Y|DifRL],LInts):-
	difer_lex0(ArgsL,ArgsR,DifLR,DifRL,LInts),!.

	
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

try_subterm_or_prec_segures([],Prec,Prec,Stat,Stat,_):-!.
try_subterm_or_prec_segures([Rule|Rules],Prec,PrecOut,Stat,StatOut,LInts):-
	Rule=..[_|[_,R]],
	is_variable(R),!,
	try_subterm_or_prec_segures(Rules,Prec,PrecOut,Stat,StatOut,LInts),!.
try_subterm_or_prec_segures([Rule|Rules],Prec,PrecOut,Stat,StatOut,LInts):-
	Rule=..[_|[L,R]],
	functor(L,F,_),functor(R,G,_),
	member212(G>F,Prec),!,
	subterm_possible_segures(L,R,LInts),!,
	try_subterm_or_prec_segures(Rules,Prec,PrecOut,Stat,StatOut,LInts),!.
try_subterm_or_prec_segures([Rule|Rules],Prec,PrecOut,Stat,StatOut,LInts):-
	Rule=..[_|[L,R]],
	functor(L,F,_), functor(R,G,_),
	exist_prec_g_ge_eq(F,G,Prec),!,
	try_subterm_or_prec_segures(Rules,Prec,PrecOut,Stat,StatOut,LInts),!.
try_subterm_or_prec_segures([Rule|Rules],Prec,PrecOut,Stat,StatOut,LInts):-
	Rule=..[_|[L,R]], 
	subterm_possible_segures(L,R,LInts),!,
	try_subterm_or_prec_segures(Rules,Prec,PrecOut,Stat,StatOut,LInts),!.
try_subterm_or_prec_segures([Rule|Rules],Prec,PrecOut,Stat,StatOut,LInts):-
	Rule=..[_|[L,R]], 
	functor(L,F,_),functor(R,G,_),
	top_segur(G,LInts),!,   % i no subterm_possible_segures
	generarF(F>=G,Prec,Prec1,Stat,Stat1),!,
	try_subterm_or_prec_segures(Rules,Prec1,PrecOut,Stat1,StatOut,LInts),!.
try_subterm_or_prec_segures([_|Rules],Prec,PrecOut,Stat,StatOut,LInts):-
	try_subterm_or_prec_segures(Rules,Prec,PrecOut,Stat,StatOut,LInts),!.


%%%%%%%%%%%%%%%

subterm_possible_segures(L,R,LInts):-
	vars_segures(R,[],LvarsR,LInts),!,	
	subterm_possible_segures1(L,R,LInts,LvarsR),!.

subterm_possible_segures1(_,_,_,[]):-!.
subterm_possible_segures1(L,R,LInts,LvarsR):-
	L=..[_|ArgsL],
	member22(Term,ArgsL),
	obtain_vars(Term,LvarsT),
	difer(LvarsR,LvarsT,[]),
	\+(en_pos_segura_Int1(R,Term,LInts)),!.  %LInts no t smbols amb 1 opc.


%%%%%%%%%%%%%%%%

	