%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  rpo13fix.pl	(03-09-02)
% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% USES : precstat, list, (ordre:is_variable)
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  check_rpo13_fix_int : es comprova que es satisf el constraint amb una versi
%			de rpo que consisteix amb cas subterme i top igual (lex o mul)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Noms es considera = sintctic. Es podria millorar per quan hi ha
% precedncia donada. En aquest cas caldria modificar la simplificaci
% quan status s cap (canviar g13cap --> caldria veure = length...)

check_rpo13_fix_int([],_,_,Stat,Stat):-!.
check_rpo13_fix_int([Const|Rest],[S,Ar],Idef,Stat,StatOut):-
	interp_const(Const,[[S,Ar,Idef]],C1),
	stat_check_call(C1,Rest,[S,Ar],Idef,Stat,StatOut).

stat_check_call(C,Rest,[S,Ar],[0|RestI],Stat,StatOut):-
	subset13_rpo_fix(C,Stat),
	check_rpo13_fix_int(Rest,[S,Ar],[0|RestI],Stat,StatOut),!.
stat_check_call(C,Rest,[S,Ar],[1|RestI],Stat,StatOut):-
	fixar_stat_S(Stat,S,Stat1),
	subset13_rpo_fix(C,Stat1),
	check_rpo13_fix_int(Rest,[S,Ar],[1|RestI],Stat1,StatOut),!.

fixar_stat_S(Stat,S,Stat):-member22([S,_],Stat),!.
fixar_stat_S(Stat,S,[[S,mul]|Stat]).
fixar_stat_S(Stat,S,[[S,lex]|Stat]).
fixar_stat_S(Stat,S,[[S,lexr]|Stat]).

subset13_rpo_fix(C,Stat):-
	C=..[>=|[L,R]], ig13_rpo_fix(L,R,Stat),!.
subset13_rpo_fix(C,Stat):-
	C=..[_|[L,R]], g13_rpo_fix(L,R,Stat),!.

ig13_rpo_fix(L,L,_):-!.
ig13_rpo_fix(L,R,Stat):-
	\+(is_variable(R)),
	L=..[F|ArgsL], R=..[F|ArgsR],
	quin_stat(F,Stat,St),  % si s cap farem la comparaci ms restrictiva
	iguals13args(ArgsL,ArgsR,St,Stat),!.


iguals13args(AL,AR,mul,Stat):-!,
	ig13mul(AL,AR,Stat).
iguals13args(AL,AR,_,Stat):-
	ig13lex(AL,AR,Stat).

ig13mul([],[],_):-!.
ig13mul([X|R1],R2,Stat):-
	treuig(X,R2,R22,Stat),!,
	ig13mul(R1,R22,Stat).

treuig(X,[Y|R],R,Stat):-
	ig13_rpo_fix(X,Y,Stat),!.
treuig(X,[Y|R],[Y|R1],Stat):-
	treuig(X,R,R1,Stat).

ig13lex([],[],_):-!.
ig13lex([X|R1],[Y|R2],Stat):-
	ig13_rpo_fix(X,Y,Stat), 
	ig13lex(R1,R2,Stat).


%%
g13_rpo_fix(L,_,_):-
	is_variable(L),!,fail.
g13_rpo_fix(L,R,_):-
	L=..[_|ArgsL],
	member_term1_list(R,ArgsL),!.
g13_rpo_fix(_,R,_):-
	is_variable(R),!,fail.
g13_rpo_fix(L,R,Stat):-
	L=..[F|ArgsL],
	R=..[F|ArgsR],
	quin_stat(F,Stat,St),
	g13_rpo_args(L,ArgsL,ArgsR,St,Stat),!.

g13_rpo_args(_,AL,AR,mul,Stat):-!,
	g13mul(AL,AR,Stat).
g13_rpo_args(L,AL,AR,lex,Stat):-!,
	g13lex(L,AL,AR,Stat).
g13_rpo_args(L,AL,AR,lexr,Stat):-!,
	g13lexr(L,AL,AR,Stat).
g13_rpo_args(_,AL,AR,cap,Stat):-
	g13cap(AL,AR,Stat),!.

g13mul([X|R1],R2,Stat):-
	treuig(X,R2,R22,Stat),!,
	g13mul(R1,R22,Stat).
g13mul(R1,R2,Stat):-
	someg13_rpo_fix(R1,R2,Stat),!.

someg13_rpo_fix([_,_],[],_):-!.
someg13_rpo_fix(R1,[Y|R2],Stat):-
	member22(X,R1), g13_rpo_fix(X,Y,Stat),!,
	someg13_rpo_fix(R1,R2,Stat).

g13lex(L,[X|R1],[Y|R2],Stat):-
	ig13_rpo_fix(X,Y,Stat),!,
	g13lex(L,R1,R2,Stat).
g13lex(_,[_|_],[],_):-!.
g13lex(T,[X|_],[Y|R1],Stat):-
	g13_rpo_fix(X,Y,Stat),!,
	g13_all_rpo_fix(T,R1,Stat).

g13_all_rpo_fix(T,[X|R],Stat):-
	g13_rpo_fix(T,X,Stat),!,
	g13_all_rpo_fix(T,R,Stat).
g13_all_rpo_fix(_,[],_):-!.

g13lexr(L,R1,R2,Stat):-
	reverse1(R1,R11),
	reverse1(R2,R22),
	g13lex(L,R11,R22,Stat).


g13cap(R1,R2,Stat):-
	g13cap(R1,R2,Stat,G), nonvar(G).

g13cap([X|R1],[Y|R2],Stat,G):-
	ig13_g13_rpo_fix(X,Y,Stat,G),!,
	g13cap(R1,R2,Stat,G).
g13cap([],[],_,_).

ig13_g13_rpo_fix(X,Y,Stat,_):-
	ig13_rpo_fix(X,Y,Stat),!.
ig13_g13_rpo_fix(X,Y,Stat,ok):-
	g13_rpo_fix(X,Y,Stat),!.

