% simp_int.pl : funcions per simplificar interpretacions (03-09-02)


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% USES: a_intrpo, list, int_rpo, (ordre:is_variable)
% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% simplify_Ints(LRules,LCycles,LInts,LIntsOut,LIntsfix,LIntsfixOut,LRulesOut,LCyclesOut)
% simplify_Ints_vars_Sat(LConst,LInts,LIntsOut)
% vars_segures(T,Lvars,LvarsOut,LInts)
% simplify_vars_rhs(Crules,LInts,LIntsOut,LIntsfix,LIntsfixOut,CrulesOut,
%			Prec,Stat,PrecOut,StatOut),
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



simplify_Ints(LRules,LCycles,LInts,LIntsOut,LIntsfix,LIntsfixOut,LRulesOut,LCyclesOut):-
	selecc_fixed_Ints_and_interpret(LInts,LInts1,LIntsfix,LIntsfix1,
		LRules,LCycles,LRules1,LCycles1),
	concat312(LRules1,LCycles1,LConst),
	simplify_Ints_vars_Sat(LConst,LInts1,LInts2),
	selecc_fixed_Ints_and_interpret(LInts2,LIntsOut,LIntsfix1,
	    LIntsfixOut,LRules1,LCycles1,LRulesOut,LCyclesOut),!.

simplify_Ints_vars_Sat(LConst,LInts0,LIntsOut):-
	simplify_Ints_vars(LConst,LInts0,LInts1),
	simplify_Ints_vars_right(LConst,LInts1,LInts2),
	simplify_Ints_proj(LConst,LInts2,LInts3),
	simplify_Ints_head(LConst,LInts3,LInts4),
	simplify_ints_stricts(LConst,LInts4,LInts5),
	sort_I(LInts0,LInts00),sort_I(LInts5,LInts55),
	ints_vars_Sat(LConst,LInts00,LInts55,LIntsOut),!.

ints_vars_Sat(_,LInts,LInts,LInts):-!.
ints_vars_Sat(LConst,_,LInts,LIntsOut):-
	simplify_Ints_vars_Sat(LConst,LInts,LIntsOut),!.


sort_I(L,LOut):-sort_I1(L,L1),sort(L1,LOut),!.

sort_I1([],[]):-!.
sort_I1([[F,Ar,L]|Rest],[[F,Ar,L1]|Rest1]):-
	sort(L,L1),
	sort_I1(Rest,Rest1).


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% simplifica interpretacions impossibles per smbols de esquerra
% degudes a les variables segures que hi ha a la dreta de cada regla

simplify_Ints_vars([],LInts,LInts):-!.
simplify_Ints_vars([Const|LConst],LInts,LIntsOut):-
	Const=..[_|[L,R]], \+(is_variable(L)),
	vars_segures(R,[],LvarsR,LInts),
	LvarsR=[_|_],!,
	contains(L,LvarsR),
	simplif_Ints_sym_left(LvarsR,L,LInts,LInts1),
	simplify_Ints_vars(LConst,LInts1,LIntsOut),!.
simplify_Ints_vars([_|LConst],LInts,LIntsOut):-
	simplify_Ints_vars(LConst,LInts,LIntsOut),!.


simplif_Ints_sym_left([],_,LInts,LInts):-!.
simplif_Ints_sym_left([Var|Rvars],Term,LInts,LIntsOut):-
	simplif_Ints_sym_leftV(Var,Term,LInts,LInts1),
	simplif_Ints_sym_left(Rvars,Term,LInts1,LIntsOut),!.

simplif_Ints_sym_leftV(Var,Term,LInts,LIntsOut):-
	Term=..[F|_],
	delete32([F,Ar,IntsF],LInts,LInts1),!,
	subpos(Term,Lpossub,Var),
	simplify_ints_possub(IntsF,Lpossub,IntsFOut),
	simplify_int_subterms(Var,Lpossub,[[F,Ar,IntsFOut]|LInts1],LIntsOut),!.
simplif_Ints_sym_leftV(Var,Term,LInts,LIntsOut):-
	subpos(Term,Lpossub,Var),
	simplify_int_subterms(Var,Lpossub,LInts,LIntsOut),!.


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% simplifica interpretacions impossibles pels smbols de dreta
% degudes a les variables que hi ha a la dreta i no a l'esquerra

simplify_Ints_vars_right([],LInts,LInts):-!.
simplify_Ints_vars_right([Const|LConst],LInts,LIntsOut):-
	Const=..[_|[L,R]],\+(is_variable(L)),
	obtain_vars_possibles(L,VL,LInts),
	obtain_vars_possibles(R,VR,LInts),
	difer(VR,VL,Vnoleft),
	Vnoleft=[_|_],!,
	simplif_Ints_sym_right(Vnoleft,R,LInts,LInts1),
	simplify_Ints_vars_right(LConst,LInts1,LIntsOut),!.
simplify_Ints_vars_right([_|LConst],LInts,LIntsOut):-
	simplify_Ints_vars_right(LConst,LInts,LIntsOut),!.


simplif_Ints_sym_right([],_,LInts,LInts):-!.
simplif_Ints_sym_right([V|Vnoleft],R,LInts,LIntsOut):-
	simplif_Ints_sym_right1(V,R,LInts,LInts1),!,
	simplif_Ints_sym_right(Vnoleft,R,LInts1,LIntsOut),!.

simplif_Ints_sym_right1(V,Term,LInts,LIntsOut):-
	Term=..[F|Args],
	delete32([F,ArF,LIntsF],LInts,LInts1),!,	 
	enumerate(Args,Argsenum),
	findall(I,pos_var_segura(I,Argsenum,V,LInts),Lposelim),!,
	del_Ints_with_poselim(LIntsF,Lposelim,LIntsF1),!,
	LIntsF1=[_|_],
	obtain_pos_segures(F,ArF,PosSeguresF,[[F,ArF,LIntsF1]]),
	obtain_args_var_segur(V,PosSeguresF,Argsenum,LArgs1),
	simplif_Ints_sym_right1list(V,LArgs1,[[F,ArF,LIntsF1]|LInts1],LIntsOut),!.
simplif_Ints_sym_right1(V,Term,LInts,LIntsOut):-
	Term=..[_|Args],		% F ja interpretat - no s a LInts
	enumerate(Args,Argsenum),
	findall(I,pos_var_segura(I,Argsenum,V,LInts),Lposelim),!,
	Lposelim=[],
	sel_args_contain_V(V,Args,LArgs1),
	simplif_Ints_sym_right1list(V,LArgs1,LInts,LIntsOut),!.


simplif_Ints_sym_right1list(_,[],LInts,LInts):-!.
simplif_Ints_sym_right1list(V,[Term|L],LInts,LIntsOut):-
	simplif_Ints_sym_right1(V,Term,LInts,LInts1),!,
	simplif_Ints_sym_right1list(V,L,LInts1,LIntsOut),!.
	

pos_var_segura(N,[[N,Arg]|_],Var,LInts):-
	conte_segur(Arg,Var,LInts).
pos_var_segura(N,[_|Args],Var,LInts):-
	pos_var_segura(N,Args,Var,LInts).

	
conte_segur(Term,Var,LInts):-
	vars_segures(Term,[],Lvars,LInts),!,
	member21(Var,Lvars),!.

del_Ints_with_poselim(LIntsF,[],LIntsF):-!.
del_Ints_with_poselim([],_,[]):-!.
del_Ints_with_poselim([IntF|LIntsF],Poselim,LIntsFOut):-
	enumerate0(IntF,IntFenum),
	member22([I,1],IntFenum),member21(I,Poselim),!,
	del_Ints_with_poselim(LIntsF,Poselim,LIntsFOut).
del_Ints_with_poselim([IntF|LIntsF],Poselim,[IntF|LIntsFOut]):-
	del_Ints_with_poselim(LIntsF,Poselim,LIntsFOut),!.

obtain_args_var_segur(_,[],_,[]):-!.
obtain_args_var_segur(_,_,[],[]):-!.
obtain_args_var_segur(Var,Pos,[[N,Arg]|Argsenum],[Arg|LArgs]):-
	member21(N,Pos),
	member_term1(Var,Arg),!,
	obtain_args_var_segur(Var,Pos,Argsenum,LArgs),!.
obtain_args_var_segur(Var,Pos,[_|Argsenum],LArgs):-
	obtain_args_var_segur(Var,Pos,Argsenum,LArgs),!.

sel_args_contain_V(_,[],[]):-!.
sel_args_contain_V(V,[Arg|Args],[Arg|LArgs]):-
	member_term1(V,Arg),!,
	sel_args_contain_V(V,Args,LArgs),!.
sel_args_contain_V(V,[_|Args],LArgs):-
	sel_args_contain_V(V,Args,LArgs),!.
	

	
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%Donat un terme T obt les variables que apareixen a la dreta
%directament o sota smbols que no les tatxaran.

vars_segures(T,L,L1,_):- is_variable(T),!, addifno(T,L,L1),!.
vars_segures(T,Lvars,Lvars,_):- T=..[_|[]],!.
vars_segures(T,Lvars0,Lvars,LInts):- 
	T=..[F|Args], length21(Args,Ar),
	obtain_pos_segures(F,Ar,Lpos,LInts),
	selecc_elems(Lpos,Args,Args1),
	vars_segures_list(Args1,Lvars0,Lvars,LInts),!.
vars_segures_list([],Lvars0,Lvars0,_):-!.
vars_segures_list([X|R],Lvars0,Lvars,LInts):-
	vars_segures(X,Lvars0,Lvars1,LInts),
	vars_segures_list(R,Lvars1,Lvars,LInts),!.

obtain_pos_segures(F,Ar,Lpos,LInts):-
	member22([F,Ar,FInts],LInts),!,
	crearlistIN(1,Ar,L1),
	map(enumerate0,FInts,FInts1),
	findall(X,pos_segura(X,L1,FInts1),Lpos),!.
obtain_pos_segures(_,Ar,Lpos,_):-
	crearlistIN(1,Ar,Lpos),!.

pos_segura(X,[X|_],L):-all_1(X,L).
pos_segura(X,[_|R],L):-pos_segura(X,R,L).

all_1(_,[]):-!.
all_1(X,[L|R]):-member22([X,1],L), all_1(X,R),!.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


% Donat un terme i una variable, retorna una llista [[pos,subterm],...]

subpos(Term,Lpossub,Var):- 
	Term=..[_|Args], enumerate(Args,A),!, 
	setof(PS,member_termV(Var,A,PS),Lpossub),!.

member_termV(Var,[[P,X]|_],[P,X]):- member_term1(Var,X).
member_termV(Var,[_|R],PS):-member_termV(Var,R,PS).


% simplify_ints_possub : simplifica les possibles interpretacions segons Lpossub.
% concretament--> aquelles interpretacions que no seleccionen cap de les 
%		posicions que es troben a la llista Lpossub.

simplify_ints_possub(IntsF,Lpossub,IntsFOut):-
	simplify_ints_possub1(IntsF,Lpossub,IntsFOut),!,
	IntsFOut=[_|_],!.

simplify_ints_possub1([],_,[]):-!.
simplify_ints_possub1([Int|RInts],L,[Int|R1]):-
	contains_some(L,Int),!, 
	simplify_ints_possub1(RInts,L,R1).
simplify_ints_possub1([_|RInts],L,R1):-
	simplify_ints_possub1(RInts,L,R1).

contains_some(L,[_|SubSel]):- 
	enumerate(SubSel,SS),
	contains_some1(L,SS),!.
contains_some1([[P,_]|_],SS):-member212([P,1],SS),!.
contains_some1([_|R],SS):-contains_some1(R,SS),!.


% simplify_int_subterms: quan a l'esquerra d'una regla col.lapsant noms hi ha
% 			un subterme que cont la variable--> simplifiquem les
%			interpretacions del smbol de cap del subterme i
%			recursivament dels seus subtermes (si s el cas:noms 1).

simplify_int_subterms(Var,[[_,Sub]],L1,LOut):-
	simp_int_subterms(Var,Sub,L1,LOut),!.
simplify_int_subterms(_,_,L,L).

simp_int_subterms(_,X,L,L):-is_variable(X).
simp_int_subterms(Var,Sub,L1,LOut):-
	Sub=..[F|_], 
	delete32([F,Ar,Fints],L1,L2),!,
	subpos(Sub,Lpossub,Var),
	simplify_ints_possub(Fints,Lpossub,Lints1),
	simplify_int_subterms(Var,Lpossub,[[F,Ar,Lints1]|L2],LOut),!.
simp_int_subterms(Var,Sub,L1,LOut):-
		% S ja t int. definida
	subpos(Sub,Lpossub,Var),
	simplify_int_subterms(Var,Lpossub,L1,LOut),!.


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% simplifica interpretacions de projecci pel terme de l'esquerra
% - portarien a una variable i > o >= i no possible interp dreta a =var
% - portarien a terme que s subterme en posici segura dreta (si >= subt. estricte).

simplify_Ints_proj([],LInts,LInts):-!.
simplify_Ints_proj([Const|LConst],LInts,LIntsOut):-
	Const=..[Op|[L,R]], \+(is_variable(L)),
	functor(L,F,_), \+(top_segur(F,LInts)),
	delete32([F,Ar,LIntsF],LInts,LInts1),
	classify_Ints(LIntsF,LIntsFp,LIntsFh),
	try_or_simplify(Op,F,Ar,LIntsFp,L,R,LInts1,LIntsFpOut),
	concat312(LIntsFpOut,LIntsFh,LIntsF1),!,LIntsF1=[_|_],
	simplify_Ints_proj(LConst,[[F,Ar,LIntsF1]|LInts1],LIntsOut),!.
simplify_Ints_proj([_|LConst],LInts,LIntsOut):-
	simplify_Ints_proj(LConst,LInts,LIntsOut),!.


classify_Ints([],[],[]):-!.
classify_Ints([[1|X]|R],L1,[[1|X]|L2]):-!,
	classify_Ints(R,L1,L2).
classify_Ints([X|R],[X|L1],L2):-
	classify_Ints(R,L1,L2).
	
% Per totes les interpretacions (que son de subterme) comprova
% que sn possibles o les elimina.
try_or_simplify(_,_,_,[],_,_,_,[]):-!.
try_or_simplify(Op,F,Ar,[IntF|LIntsFp],L,R,LInts,LIntsFpOut):-
	selecc_fixed_ints(LInts,LInts1,LIntsfix),
	LInts2=[[F,Ar,IntF]|LIntsfix],
	interp_term(L,LInts2,L1),
	interp_term(R,LInts2,R1),
	needs_simplify(L1,R1,Op,LInts1),!,
	try_or_simplify(Op,F,Ar,LIntsFp,L,R,LInts,LIntsFpOut).
try_or_simplify(Op,F,Ar,[IntF|LIntsFp],L,R,LInts,[IntF|LIntsFpOut]):-
	try_or_simplify(Op,F,Ar,LIntsFp,L,R,LInts,LIntsFpOut).


needs_simplify(L,_,'>',_):-is_variable(L),!.
needs_simplify(L,R,'>=',LInts):-
	is_variable(L),!,
	no_poss_Int_to_var(L,R,LInts),!.
needs_simplify(L,R,Op,LInts):-
	no_accept(Op,L,R,LInts).

% cert si L s subterme de R en pos segura 
% (estricte/no segons >= o >)
no_accept('>',L,R,LInts):-
	en_pos_segura_Int(R,L,LInts),!.   
no_accept('>=',L,R,LInts):-
	strict_subt_segura(R,L,LInts),!.

no_poss_Int_to_var(V,V,_):-!,fail.
no_poss_Int_to_var(_,V1,_):- is_variable(V1),!.
no_poss_Int_to_var(_,Term,LInts):-
	functor(Term,F,_),
	top_segur(F,LInts),!.
no_poss_Int_to_var(Var,Term,LInts):-
	subpos(Term,Lpossub,Var),
	Term=..[F|_],
	pos_impos(F,LInts,Limpos),
	elim_impos(Lpossub,Limpos,SubtermsVar),
	no_poss_Int_to_var_list(Var,SubtermsVar,LInts),!.

no_poss_Int_to_var_list(_,[],_):-!.
no_poss_Int_to_var_list(Var,[S|Rest],LInts):-
	no_poss_Int_to_var(Var,S,LInts),
	no_poss_Int_to_var_list(Var,Rest,LInts),!.
	
pos_impos(F,LInts,Limpos):-
	member22([F,Ar,LIntsF],LInts),!,
	crearlistIN(1,Ar,L),
	map(tail,LIntsF,LIntsSub),
	obt_pos_impos(L,LIntsSub,Limpos),!.
pos_impos(_,_,[]).

obt_pos_impos([],_,[]):-!.
obt_pos_impos([X|L],LIntsSub,[X|Limpos]):-
	map(head,LIntsSub,Q),
	alllist_K(Q,0),!,
	map(tail,LIntsSub,LIntsSub1),
	obt_pos_impos(L,LIntsSub1,Limpos).
obt_pos_impos([_|L],LIntsSub,Limpos):-
	map(tail,LIntsSub,LIntsSub1),
	obt_pos_impos(L,LIntsSub1,Limpos).



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% simplifica interpretacions pel terme de la dreta
% - que farien que L passi a ser subterme segur de R
% - 

simplify_Ints_head([],LInts,LInts):-!.
simplify_Ints_head([Const|LConst],LInts,LIntsOut):-
	Const=..[Op|[L,R]], \+(is_variable(R)), 
	member_term_igInt(L,R,LInts),!,	
	\+(no_accept(Op,L,R,LInts)),!,   % s subt. per no en pos. segura
	simplify_Ints_head1(Const,LInts,LInts1),
	simplify_Ints_head(LConst,LInts1,LIntsOut),!.
simplify_Ints_head([_|LConst],LInts,LIntsOut):-
	simplify_Ints_head(LConst,LInts,LIntsOut),!.


simplify_Ints_head1(Const,LInts,LIntsOut):-
	Const=..[Op|[L,R]], R=..[G|_],
	delete32([G,Ar,LIntsG],LInts,LInts1),
	LIntsG=[_,_|_],!,       % hi ha ms d'una opci 
	compl_Op(Op,Op2),       % quedar-se amb les de head portaria a 
	R2=..[Op2|[R,L]],	% trivial el contrari (L subt pos segura de R)	
	simpl_Ints_triv(R2,G,Ar,LIntsG,LInts1,LIntsG1),!,
	LIntsG1=[_|_],
	LIntsOut=[[G,Ar,LIntsG1]|LInts1],!.
simplify_Ints_head1(Const,LInts,LIntsOut):-
	Const=..[Op|[L,R]], 	% una sola opc. de proj. 
	R=..[G|_],		
	member21([G,Ar,[Intp]],LInts),Intp=[0|_],!,
  	interp_terms([L,R],[[G,Ar,Intp]],[L1,R1]),
	( ( member_term_igInt(L1,R1,LInts),!,  % primer subt. de segon desprs d'int	
	    \+(no_accept(Op,L1,R1,LInts)),!,   % s subt. per no en pos. segura (sin fail)
	    Const2=..[Op|[L1,R1]],
	    simplify_Ints_head1(Const2,LInts,LIntsOut) 
	  )  
          ;
	  LIntsOut=LInts),!.
simplify_Ints_head1(Const,LInts,LIntsOut):-
	Const=..[_|[L,R]], R=..[G|_],	% ja interpretat o noms una opci de head
	(
	 ( member22([G,Ar,[IntG]],LInts),!,
   	   interp_terms([L,R],[[G,Ar,IntG]],[L1,R1])
	 ) 
 	 ;
	 ( L1=L , R1=R )
	),
	R1=..[G|ArgsR1],!,
	obt_subt_contain_igInt(ArgsR1,L,ArgsOut,LInts),
	simplify_Ints_head1list(ArgsOut,L,LInts,LIntsOut),!.



simpl_Ints_triv(_,_,_,[],_,[]):-!.
simpl_Ints_triv(Cons,G,Ar,[IntG|LIntsG],LInts,LIntsGOut):-
	trivial(Cons,[[G,Ar,[IntG]]|LInts]),!,
	simpl_Ints_triv(Cons,G,Ar,LIntsG,LInts,LIntsGOut),!.
simpl_Ints_triv(Cons,G,Ar,[IntG|LIntsG],LInts,[IntG|LIntsGOut]):-
	simpl_Ints_triv(Cons,G,Ar,LIntsG,LInts,LIntsGOut),!.




obt_subt_contain_igInt([],_,[],_):-!.
obt_subt_contain_igInt([A|Args],L,[A|ArgsOut],LInts):-
	member_term_igInt(L,A,LInts),!,
	\+(no_accept('>',L,A,LInts)),
	obt_subt_contain_igInt(Args,L,ArgsOut,LInts),!.	  
obt_subt_contain_igInt([_|Args],L,ArgsOut,LInts):-
	obt_subt_contain_igInt(Args,L,ArgsOut,LInts),!.

	
simplify_Ints_head1list([],_,LInts,LInts):-!.
simplify_Ints_head1list([A|Args],L,LInts,LIntsOut):-
	simplify_Ints_head1(L>A,LInts,LInts1),
	simplify_Ints_head1list(Args,L,LInts1,LIntsOut),!.



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% simplifica interpretacions quan mateix head ( cas > ).

simplify_ints_stricts([],L,L):-!.
simplify_ints_stricts([C|Rest],LI1,LIOut):-
	C=..[>|[Left,Right]],
	Left=..[F|ArgsL], 
	Right=..[F|ArgsR],
	delete32([F,Ar,LInts],LI1,LI2),
	crearlistNK(Ar,0,L),
	delete0312([1|L],LInts,L1),
	pos_sub_or_eq(ArgsL,ArgsR,Ar,Lpos,LI1),
	elimina(Lpos,L1,L2),!, \+(L2=[]),
	simplify_ints_stricts(Rest,[[F,Ar,L2]|LI2],LIOut),!.
simplify_ints_stricts([_|Rest],LI1,LIOut):-
	simplify_ints_stricts(Rest,LI1,LIOut).


pos_sub_or_eq(ArgsL,ArgsR,Ar,Lpos,LInts):- 
	crearlistIN(1,Ar,Lenum),
	pos_s_eq(Lenum,ArgsL,ArgsR,Lpos,LInts).
pos_s_eq([],[],[],[],_).
pos_s_eq([N|Rn],[S|Rs],[T|Rt],[N|R],LInts):-
	pos_a_considerar(S,T,LInts),!,
	pos_s_eq(Rn,Rs,Rt,R,LInts).
pos_s_eq([_|Rn],[_|Rs],[_|Rt],R,LInts):-
	pos_s_eq(Rn,Rs,Rt,R,LInts).
	
pos_a_considerar(S,_,_):-is_variable(S),!.
pos_a_considerar(S,T,LInts):-
	en_pos_segura_Int(T,S,LInts),!.
%	member_term1(S,T),
%	en_pos_segura(T,S,LInts),!.

elimina(_,[],[]):-!.
elimina(Lpos,[Int|LInt],[Int|L]):-    % ens quedem amb tot 
	alllist_K(Int,1),!,
	elimina(Lpos,LInt,L).		
elimina(Lpos,[Int|LInt],[Int|L]):-    % almenys ens quedem amb 1 possible
	enumerate(Int,0,Intpos),
	member22([P,1],Intpos),P>0,
	\+(member212(P,Lpos)),!,
	elimina(Lpos,LInt,L).		
elimina(Lpos,[_|LInt],L):-		
	elimina(Lpos,LInt,L).		


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% selecciona les interpretacions que ja noms tenen una opcio
% interpreta regles i cicles i les afegeix a LIntsfix
% . Si els que noms tenen una opci sn per smbols renombrats 
% i la interpretaci s de projecci, es treu tot el set de cicles 
% i es posa a regles.

selecc_fixed_Ints_and_interpret(LInts,LIntsOut,LIntsfix,LIntsfixOut,
				Rules,Cycles,RulesOut,CyclesOut):-
	selecc_fixed_ints(LInts,LIntsOut,LIntsfixnous),
	((LIntsfixnous=[],!,RulesOut=Rules,CyclesOut=Cycles,LIntsfixOut=LIntsfix)
  	 ;
	 ( renamed(Ren),
	   separ_proj_renamed(Ren,LIntsfixnous,LIntsprojren),
	   act_interpret(Rules,Cycles,LIntsfixnous,LIntsprojren,RulesOut,CyclesOut),
	   concat312(LIntsfix,LIntsfixnous,LIntsfixOut)
	 )
	),!.

selecc_fixed_ints([],[],[]):-!.
selecc_fixed_ints([X|Rest],LIntsOut,[[F,Ar,L]|LfixedOut]):-
	X=[F,Ar,[L]],!,
	selecc_fixed_ints(Rest,LIntsOut,LfixedOut),!.
selecc_fixed_ints([X|Rest],[X|LIntsOut],LfixedOut):-
	selecc_fixed_ints(Rest,LIntsOut,LfixedOut),!.

%selecciona les interpretacions fixes que sn tot 1's --> no cal interpretar
selecc_fixed1_ints([],L,L,Lf,Lf):-!.
selecc_fixed1_ints([X|Rest],LInts,LInts1,Lfixed,LfixedOut):-
	X=[F,Ar,[L]], alllist_K(L,1),!,
	selecc_fixed1_ints(Rest,LInts,LInts1,[[F,Ar,L]|Lfixed],LfixedOut),!.
selecc_fixed1_ints([X|Rest],LInts,LInts1,Lfixed,LfixedOut):-
	selecc_fixed1_ints(Rest,[X|LInts],LInts1,Lfixed,LfixedOut),!.


separ_proj_renamed(Ren,LIntsfix,LIntsRenProj):-
	map(head2,LIntsfix,Symbs),
	inter(Ren,Symbs,Renfix),
	sep_proj_renamed(Renfix,LIntsfix,LIntsRenProj),!.
	
sep_proj_renamed(_,[],[]):-!.
sep_proj_renamed(Renfix,[[_,_,[1|_]]|LIntsfix],LIntsRenProj):-!,
	sep_proj_renamed(Renfix,LIntsfix,LIntsRenProj),!.
sep_proj_renamed(Renfix,[[F,Ar,IntF]|LIntsfix],[[F,Ar,IntF]|LIntsRenProj]):-
	member212([F,Ar],Renfix),!,
	sep_proj_renamed(Renfix,LIntsfix,LIntsRenProj),!.
sep_proj_renamed(Renfix,[_|LIntsfix],LIntsRenProj):-
	sep_proj_renamed(Renfix,LIntsfix,LIntsRenProj),!.
	


act_interpret(Rules,Cycles,LIntsfixnous,[],RulesOut,CyclesOut):-
	interpret(Rules,LIntsfixnous,[],RulesOut),
	interpret(Cycles,LIntsfixnous,[],CyclesOut),!.
act_interpret(Rules,Cycles,LIntsfixnous,LIntsprojren,RulesOut,CyclesOut):-
	map(head,LIntsprojren,Symbs),
	obt_set_cycles(Cycles,Symbs,_,Cset,Restcycles),!,
	concat312(Rules,Cset,NRules),
	interpret(NRules,LIntsfixnous,[],RulesOut),
	interpret(Restcycles,LIntsfixnous,[],CyclesOut),!.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Per totes les variables que poden quedar a la dreta i no apareixen a 
%	l'esquerra es fixa una interpretaci que les elimina (opc. backtrack.)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

simplify_vars_rhs(LConst,LInts,LIntsOut,LIntsfix,LIntsfixOut,LConstOut,
				Prec,Stat,PrecOut,StatOut):-
	member22(Const,LConst),
	Const=..[_|[L,R]],
	obtain_vars_possibles(R,VR,LInts),
	obtain_vars_possibles(L,VL,LInts),
	difer(VR,VL,Lvars), \+(Lvars=[]),!,
	obtain_posic_vars(Lvars,R,Lcamins,LInts),
	findall(Lsympos,selecc_pos_elim(Lcamins,[],Lsympos),L_all_selecc),
	elim_subsum(L_all_selecc,[],Lselecs),!,
	member22(Lsympossel,Lselecs),
	aplicar_elim(Lsympossel,LInts,LInts1),
	simplify_and_check1(LConst,LInts1,LIntsfix,LConst2,LInts2,LIntsfix2,Prec,Stat,Prec0,Stat0),
	set_int_leftisvar1(LConst2,LInts2,LIntsfix2,LConst3,LInts3,LIntsfix3,Prec0,Stat0,Prec1,Stat1),
	simplify_vars_rhs(LConst3,LInts3,LIntsOut,LIntsfix3,LIntsfixOut,LConstOut,
					Prec1,Stat1,PrecOut,StatOut).
simplify_vars_rhs(LConst,L,L,Lf,Lf,LConst,P,S,P,S).


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% obt les variables de Term que s possible que una vegada interpretat
% el terme hi siguin --> hi ha alguna interpretaci a LInts que la mant.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

obtain_vars_possibles(Term,L,LInts):-
	obtain_vars_possibles(Term,[],L,LInts).

obtain_vars_possibles([],L,L,_):-!.
obtain_vars_possibles([X|R],L,Lvars,LInts):-!,
	obtain_vars_possibles(X,L,L1,LInts),
	obtain_vars_possibles(R,L1,Lvars,LInts),!.
obtain_vars_possibles(T,L,Lvars,_):-
	is_variable(T),!,
	addifno(T,L,Lvars).
obtain_vars_possibles(T,L,Lvars,LInts):-
	T=..[F|Args],
	member21([F,_,LIntsF],LInts),!,
	args_possibles(Args,LIntsF,Args1,no),
	obtain_vars_possibles(Args1,L,Lvars,LInts),!.
obtain_vars_possibles(T,L,Lvars,LInts):-  
	T=..[_|Args],		%ja interpretat --> tots els args sn possibles
	obtain_vars_possibles(Args,L,Lvars,LInts),!.


%% Retorna els arguments possibles segons LIntsF. Si Enum=yes retorna els
%% arguments enumerats.
args_possibles(Args,LIntsF,Args1,Enum):-
	enumerate(Args,ArgsEnum),
	map(enumerate0,LIntsF,LIntsFEnum),
	args_possib(ArgsEnum,LIntsFEnum,Args1,Enum).

args_possib([[P,S]|ArgsEnum],LIntsFEnum,[S|Args1],no):-
	member22(L,LIntsFEnum),
	member22([P,1],L),!,
	args_possib(ArgsEnum,LIntsFEnum,Args1,no),!.
args_possib([[P,S]|ArgsEnum],LIntsFEnum,[[P,S]|Args1],yes):-
	member22(L,LIntsFEnum),
	member22([P,1],L),!,
	args_possib(ArgsEnum,LIntsFEnum,Args1,yes),!.
args_possib([_|ArgsEnum],LIntsFEnum,Args1,Enum):-
	args_possib(ArgsEnum,LIntsFEnum,Args1,Enum).
args_possib([],_,[],_):-!.


%%%%%%%%%
obtain_posic_vars(Lvars,Term,Lcamins,LInts):-
	obtain_posic_vars(Lvars,Term,[],Lcamins0,LInts),
	map(sort,Lcamins0,Lcamins1),
	elim_subsum(Lcamins1,[],Lcamins),!.

obtain_posic_vars([],_,L,L,_):-!.
obtain_posic_vars([Var|Lvars],Term,L0,Lcamins,LInts):-
	obtain_posic_var(Var,Term,L1,LInts),
	concat312(L0,L1,L),
	obtain_posic_vars(Lvars,Term,L,Lcamins,LInts),!.

obtain_posic_var(Var,Var,[[]],_):-!.
obtain_posic_var(Var,Term,Lcamins,LInts):-
	Term=..[F|Args],
	member21([F,_,LIntsF],LInts),!,
	args_possibles(Args,LIntsF,Argsenum,yes),
	obt_posic_var(Var,F,Argsenum,[],Lcamins,LInts).
obtain_posic_var(Var,Term,Lcamins,LInts):-
	Term=..[F|Args],		% ja interpretat -> tots possibles
	enumerate(Args,Argsenum),
	obt_posic_var(Var,F,Argsenum,[],Lcamins,LInts).
	
obt_posic_var(Var,F,[[P,S]|Argsenum],L,LOut,LInts):-
	member_term1(Var,S),!,
	obtain_posic_var(Var,S,Lc1,LInts),
	map(addifno,[F,P],Lc1,Lc2),
	concat312(L,Lc2,Lc),
	obt_posic_var(Var,F,Argsenum,Lc,LOut,LInts),!.	
obt_posic_var(Var,F,[_|Argsenum],L,LOut,LInts):-
	obt_posic_var(Var,F,Argsenum,L,LOut,LInts),!.
obt_posic_var(_,_,[],L,L,_):-!.



%-----------------------
% elimina subsumits on una llista l1 subsumeix una llista l2 si
% l1 est inclosa a l2 (--> ens quedem amb la ms curta)

elim_subsum([],L,L):-!.
elim_subsum([L|R],Lc,Lcamins):-
	concat312(R,Lc,Lsub),
	subsum(L,Lsub),!,
	elim_subsum(R,Lc,Lcamins),!.
elim_subsum([L|R],Lc,Lcamins):-
	elim_subsum(R,[L|Lc],Lcamins),!.

subsum(L,LL):-  member22(L1,LL),
		difer(L1,L,[]),!.


%-----------------------
selecc_pos_elim([],L,LOut):-sort(L,LOut).
selecc_pos_elim([Cami|Lcamins],L,Lsympos):-
	member22([Sym,Pos],Cami),
	selecc_pos_elim(Lcamins,[[Sym,Pos]|L],Lsympos).   

%-----------------

aplicar_elim([],L,L):-!.
aplicar_elim([[F,P]|Lsympos],LInts,LIntsOut):-
	delete32([F,Ar,LIntsF],LInts,LInts1),
	remove_ints_sub(P,LIntsF,LIntsF1),!,
	\+(LIntsF1=[]),
	aplicar_elim(Lsympos,[[F,Ar,LIntsF1]|LInts1],LIntsOut),!.


remove_ints_sub(_,[],[]):-!.
remove_ints_sub(P,[IntF|RestIntF],[IntF|RestIntFOut]):-
	enumerate(IntF,0,IntFenum),
	member22([P,0],IntFenum),!,
	remove_ints_sub(P,RestIntF,RestIntFOut),!.
remove_ints_sub(P,[_|RestIntF],RestIntFOut):-
	remove_ints_sub(P,RestIntF,RestIntFOut),!.





